''' <summary>
''' Ein ToolStripDropDownButton zum Auswhlen von Directories
''' </summary>
''' <remarks></remarks>
Public Class DirectoryToolStripDropDownButton
    Inherits ToolStripDropDownButton

#Region " Private Felder / Events "

    ' Liste mit enthaltenen ToolStripMenuItems
    Private m_Items As New List(Of DirectoryToolStripMenuItem)
    ' aktuell ausgewhltes Item
    Private m_SelectedItem As DirectoryToolStripMenuItem = Nothing
    ' Event bei nderung des ausgehlten Items
    Public Event SelectedPath_Changed()

#End Region

#Region " Properties "

    ''' <summary>
    ''' Gibt den aktuell ausgewhlten Pfad zurck
    ''' </summary>
    ReadOnly Property SelectedPath() As String
        Get
            If m_SelectedItem Is Nothing Then
                Return ""
            Else
                Return m_SelectedItem.Path
            End If
        End Get
    End Property

#End Region

#Region " Konstruktor "
    Public Sub New()
        Me.DisplayStyle = ToolStripItemDisplayStyle.ImageAndText
        ' Liste mit anzeigbaren Directories erstellen
        Dim Liste As List(Of String) = GetDirectories()
        ' Fr jeden Pfad in der Liste ein ToolStripMenuItem hinzufgen
        For Each txt As String In Liste
            Dim Item As New DirectoryToolStripMenuItem(txt)
            ' Eventhandler fr Klick-Event auf dieses Item
            AddHandler Item.Click, AddressOf Item_Click
            Me.DropDownItems.Add(Item)
            ' Das neue Item zur internen Liste hinzufgen
            m_Items.Add(Item)
        Next
        Me.Image = My.Resources.openfolder
        ' das erste verfgbare Item zum selektierten machen
        SetSelected(m_Items(0), False)
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' liefert eine Liste mit Pfaden fr eine Reihe Systemordner 
    ''' </summary>
    Private Function GetDirectories() As List(Of String)
        Dim Result As New List(Of String)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.Desktop), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.DesktopDirectory), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.Personal), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.MyMusic), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.MyPictures), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.ProgramFiles), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.Programs), Result)
        AddToList(Environment.GetFolderPath(Environment.SpecialFolder.InternetCache), Result)

        Dim ExitingDirs() As String = Result.ToArray
        For Each txt As String In ExitingDirs
            AddToList(IO.Directory.GetParent(txt).FullName, Result)
        Next
        Return Result
    End Function

    ''' <summary>
    ''' fgt einen Directory-Pfad, wenn noch nicht enthalten, einer Liste an
    ''' </summary>
    Private Sub AddToList(ByVal Text As String, ByVal List As List(Of String))
        If Text <> "" Then
            Dim PathText As String = IO.Path.GetFullPath(Text)
            If IO.Directory.Exists(PathText) Then
                If Not List.Contains(PathText) Then
                    List.Add(PathText)
                End If
            End If
        End If
    End Sub

    ''' <summary>
    ''' Eventhandler fr das Click-Event fr ein enthaltenes ToolStripMenuItem
    ''' </summary>
    Private Sub Item_Click(ByVal sender As Object, ByVal e As System.EventArgs)
        SetSelected(CType(sender, DirectoryToolStripMenuItem), True)
    End Sub

    ''' <summary>
    ''' legt das aktuell ausgewhlte ToolStripMenuItem fest
    ''' </summary>
    ''' <param name="Item">ausgewhltes Item</param>
    ''' <param name="withEvent">True, wenn ein Event fr nderung geliefert werden soll</param>
    Private Sub SetSelected(ByVal Item As DirectoryToolStripMenuItem, ByVal withEvent As Boolean)
        If m_SelectedItem IsNot Item Then
            m_SelectedItem = Item
            For Each Itm As DirectoryToolStripMenuItem In m_Items
                Itm.Image = My.Resources.closedfolder
            Next
            m_SelectedItem.Image = My.Resources.openfolder
            Me.Text = m_SelectedItem.Text
            If withEvent Then RaiseEvent SelectedPath_Changed()
        End If
    End Sub

#End Region

#Region " DirectoryToolStripMenuItem "
    ''' <summary>
    ''' Ein ToolStripMenuItem zum Anzeigen eines einzelnen Directory
    ''' </summary>
    ''' <remarks></remarks>
    Private Class DirectoryToolStripMenuItem
        Inherits ToolStripMenuItem
        Private m_Path As String

        ''' <summary>
        ''' Pfad zum zugeordneten Directory
        ''' </summary>
        ReadOnly Property Path() As String
            Get
                Return m_Path
            End Get
        End Property

        ''' <summary>
        ''' Konstruktor mit Angabe eines Directory-Pfades
        ''' </summary>
        Public Sub New(ByVal Path As String)
            m_Path = Path
            Dim Txt As String = IO.Path.GetFileName(m_Path)
            If Txt = "" Then
                ' wenn es sich um einen Root-Pfad handelt
                Txt = m_Path
            End If
            Me.Text = Txt
            Me.Image = My.Resources.closedfolder
        End Sub
    End Class
#End Region

End Class
