Imports System.ComponentModel
''' <summary>
''' Ein TreeView-Control zum Anzeigen einer Directory-Struktur
''' </summary>
Public Class FolderTreeView
    Inherits TreeView

#Region " Private Felder "
    ' ImageList mit Ordner-Bildchen
    Private m_IL As ImageList

    ' Directory, das als oberster Ordner angezeigt werden soll
    Private m_TopDirectoryPath As String = ""
    ' Zeichenfolge, die die Directory-Struktur vom RootPath (z.B. C:\) bis 
    ' zum Parent von TopDirectoryPath enthlt
    Private m_Root As String
    ' aktuell ausgewhlter Pfad
    Private m_SelectedPath As String = ""
    ' Event bei nderung des aktuell ausgewhlten Pfades
    Public Event SelectedPath_Changed()
#End Region

#Region " Properties "
    <Category("Appearance"), Description("Aktuell ausgewhlter Pfad.")> _
    Property SelectedPath() As String
        Get
            Return m_SelectedPath
        End Get
        Private Set(ByVal value As String)

            Dim NewValue As String = IO.Path.GetFullPath(value)
            If m_SelectedPath <> NewValue Then
                m_SelectedPath = NewValue
                ' nderungs-Event
                RaiseEvent SelectedPath_Changed()
            End If
        End Set
    End Property

    <Category("Appearance"), Description("Gibt den Pfad zum ersten angezeigten Directory zurck oder legt diesen fest.")> _
    Property TopDirectoryPath() As String
        Get
            Return m_TopDirectoryPath
        End Get
        Set(ByVal value As String)
            If IO.Directory.Exists(value) AndAlso m_TopDirectoryPath <> value Then
                m_TopDirectoryPath = value
                ' Directory-Struktur neu anzeigen
                FirstScan()
            End If
        End Set
    End Property
#End Region

#Region " Konstruktor "
    Public Sub New()
        ' Pfad-Separator explizit festlegen
        Me.PathSeparator = IO.Path.DirectorySeparatorChar
        ' ImageList mit Bildchen fr geffneten und geschlossenen Ordner anlegen
        m_IL = New ImageList
        m_IL.ColorDepth = ColorDepth.Depth32Bit
        m_IL.ImageSize = New Size(16, 16)
        m_IL.Images.Add(My.Resources.closedfolder)
        m_IL.Images.Add(My.Resources.openfolder)
        Me.ImageList = m_IL
        ' fr selektierte Items wird ein geffneter Ordner angezeigt
        Me.SelectedImageIndex = 1
        ' auch wenn das Control keinen Fokus hat, soll der ausgewhlte 
        ' Ordner erkennbar sein
        Me.HideSelection = False
    End Sub
#End Region

#Region " Methoden "
    Private Sub FirstScan()
        ' passiert bei Ersten Festlegen des Top-Directory
        ' Hier werden die zuoberst sichtbaren Strukturknoten festgelegt.
        ' (Dazu werden rekursiv brigens zwei Ebenen der Directory-Struktur untersucht)

        Try
            ' gegebenenfalls vorhandene TreeNodes entfernen
            Me.Nodes.Clear()

            ' Root bestimmen
            Dim RootPath As String = IO.Path.GetPathRoot(m_TopDirectoryPath)
            Dim NewNodeText As String
            ' Es kann passieren, da als TopDirectory ein RootDirectory 
            ' ausgewhlt wurde. Weit ich fr
            ' SelectedPath = m_Root & e.Node.FullPath verwenden will,
            ' muss hier m_Root und der Text fr den RootNode besonders 
            ' festgelegt werden
            If RootPath = m_TopDirectoryPath Then
                m_Root = ""
                NewNodeText = m_TopDirectoryPath
            Else
                NewNodeText = IO.Path.GetFileName(m_TopDirectoryPath)
                m_Root = IO.Directory.GetParent(m_TopDirectoryPath).FullName & IO.Path.DirectorySeparatorChar
            End If

            ' obersten TreeNode erstellen
            Dim NewNode As New TreeNode(NewNodeText)
            Me.Nodes.Add(NewNode)
            ' die SubDirectories dieses obersten Nodes untersuchen
            CheckNode(NewNode)
            ' obersten TreeNode Selektieren
            Me.SelectedNode = NewNode
            ' vor Expand werden brigens dann die SubDirectories fr die 
            ' ChildNodes untersucht
            NewNode.Expand()
        Catch ex As Exception
            ' bei einem Fehler (z.B. IO.Exception) kann kein SelectedPath 
            ' angezeigt werden
            SelectedPath = ""
        End Try

    End Sub

    Private Sub FolderTreeView_AfterSelect(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewEventArgs) Handles Me.AfterSelect
        ' nach Selektieren eines Nodes Anderung fr SelectedPath-Property
        ' (In der Folge kommt das Changed-Event)
        SelectedPath = m_Root & e.Node.FullPath
    End Sub


    Private Sub FolderTreeView_BeforeCollapse(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewCancelEventArgs) Handles Me.BeforeCollapse
        ' Um Arbeitsspeicher zu sparen, werden die Children der ChildNodes 
        ' dynamisch entfernt (und spter bei Bedarf wieder hinzu gefgt)
        e.Node.ImageIndex = 0
        For Each Nd As TreeNode In e.Node.Nodes
            Nd.Nodes.Clear()
        Next
    End Sub


    Private Sub FolderTreeView_BeforeExpand(ByVal sender As Object, ByVal e As System.Windows.Forms.TreeViewCancelEventArgs) Handles Me.BeforeExpand
        ' Children der ChildNodes untersuchen
        For Each Nd As TreeNode In e.Node.Nodes
            CheckNode(Nd)
        Next

        e.Node.ImageIndex = 1
        ' Beim ffnen wird dieser TreeNode gleichzeitig selektiert
        Me.SelectedNode = e.Node
    End Sub


    Private Sub CheckNode(ByVal node As TreeNode)
        ' Untersucht die Children eines TreeNode auf Existenz von untergeordneten
        ' Directories und fgt bei Bedarf neue ChildNodes hinzu
        Dim Path As String = m_Root & node.FullPath
        Try
            For Each Txt As String In IO.Directory.GetDirectories(Path)
                node.Nodes.Add(IO.Path.GetFileName(Txt))
            Next
        Catch ex As Exception

        End Try
    End Sub
#End Region

End Class

