''' <summary>
''' Eine generische Liste um fr meine Zwecke SelectedItems zu sammeln und falls notwendig
''' ein Changed-Event zu liefern
''' </summary>
Public Class SelectionListe(Of T)
    Inherits List(Of T)
    Public Event Changed()
    Public Shadows Sub Add(ByVal item As T)
        If Not Me.Contains(item) Then
            MyBase.Add(item)
            RaiseEvent Changed()
        End If
    End Sub
    Public Shadows Function Remove(ByVal item As T) As Boolean
        Dim Result As Boolean = MyBase.Remove(item)
        If Result = True Then
            RaiseEvent Changed()
        End If
        Return Result
    End Function
    Public Shadows Sub AddRange(ByVal collection As IEnumerable(Of T))
        Dim WasChanged As Boolean = False
        For Each Item As T In collection
            If Not Me.Contains(Item) Then
                MyBase.Add(Item)
                WasChanged = True
            End If
        Next
        If WasChanged Then
            RaiseEvent Changed()
        End If
    End Sub
    Public Shadows Sub Remove(ByVal collection As IEnumerable(Of T))
        Dim WasChanged As Boolean = False
        For Each Item As T In collection
            Dim Result As Boolean = MyBase.Remove(Item)
            If Result = True Then
                WasChanged = True
            End If
        Next
        If WasChanged Then
            RaiseEvent Changed()
        End If
    End Sub
    Public Shadows Sub Clear(ByVal Raiseevents As Boolean)
        Dim WasChanged As Boolean = False
        If Me.Count > 0 Then
            WasChanged = True
        End If
        MyBase.Clear()
        If WasChanged Then
            If Raiseevents Then RaiseEvent Changed()
        End If
    End Sub
    Public Sub SetSelectedItem(ByVal Item As T)
        MyBase.Clear()
        MyBase.Add(Item)
        RaiseEvent Changed()
    End Sub
    Public Sub SetSelectedItems(ByVal collection As IEnumerable(Of T))
        MyBase.Clear()
        MyBase.AddRange(collection)
        RaiseEvent Changed()
    End Sub
End Class
