Imports System.ComponentModel
''' <summary>
''' Control, das Bildinhalte eines Directory als Thumbnails darstellen kann
''' </summary>
Public Class ThumbControl
    Inherits ScrollableControl

#Region " Private Felder "

    ' Directory, dessen Dateien hier angezeigt werden
    Private m_Path As String = ""
    ' Breite einer vertikalen ScrollBar
    Private m_ScrollBarWidth As Integer
    ' Pen zum Hervorheben ausgewhlter Items
    Private m_SelectionPen As Pen

    ' Breite eines angezeigten Thumbnails
    Private m_ImageWidth As Integer = 100
    ' Distanz zwischen zwei Thumbnails
    Private m_InnerDistance As Integer = 10

    ' Liste mit Painter-Objekten, die jeweils in der Lage sind, eine einzelne 
    ' Thumbnail-Datei zu zeichnen
    Private m_Painters As New List(Of ThumbPainter)
    ' Liste, die nur die Painter enthlt, die aktuell gerade im sichtbaren Bereich sind
    ' (unsichtbare Painter brauchen nichts zu zeichnen)
    Private m_VisiblePainters As New List(Of ThumbPainter)

    ' Array, das die aktuell angezeigten Pfade enthlt
    Private m_Pfade() As ThumbPath
    ' aktuell fr die angezeigten Dateien verwendete Sortierreihenfolge
    Private m_Sortierung As ThumbSortierTypen = ThumbSortierTypen.Normal

    ' Liste mit selektierten ThumbPainter-Objekten
    Private WithEvents m_SelectedPainters As New SelectionListe(Of ThumbPainter)
    ' zuletzt selektiertes ThumbPainter-Objekt
    Private m_lastSelectedPainter As ThumbPainter
#End Region

#Region " Properties "

    <Category("Appearance"), Description("Gibt die Sortierung der angezeigten Dateien zurck oder legt diese fest.")> _
    Property Sortierung() As ThumbSortierTypen
        Get
            Return m_Sortierung
        End Get
        Set(ByVal value As ThumbSortierTypen)
            If m_Sortierung <> value Then
                m_Sortierung = value

                Neu_Sortieren()

                Me.VerticalScroll.Value = 0
                ' Painter-Positionen und sichtbare Painter neu festlegen und Anzeige aktualisieren
                RefreshVisiblePainters()
                m_lastSelectedPainter = Nothing
            End If
        End Set
    End Property

    <Category("Appearance"), Description("Gibt den Pfad zum Directory, dessen Dateien angezeigt werden zurck oder legt diesen fest.")> _
    Property Path() As String
        Get
            Return m_Path
        End Get
        Set(ByVal value As String)
            If m_Path <> value AndAlso IO.Directory.Exists(value) Then
                m_Path = value
                ' Inhalte des neuen Pfades anzeigen
                DisplayNewPath()
            End If
        End Set
    End Property

    <Category("Appearance"), Description("Gibt die Kantenlnge der angezeigten Thumbnails zurck oder legt diese fest.")> _
    Property ImageWidth() As Integer
        Get
            Return m_ImageWidth
        End Get
        Set(ByVal value As Integer)
            Dim TmpValue As Integer = Math.Max(Math.Min(value, 400), 100)
            If m_ImageWidth <> TmpValue Then
                m_ImageWidth = TmpValue

                ' Hier mssen gegebenenfalls vorhandene Bitmaps entfernt werden
                For Each Ptr As ThumbPainter In m_Painters
                    Ptr.DisposeBitmap()
                Next
                ' Painter-Positionen und sichtbare Painter neu festlegen und Anzeige aktualisieren
                RefreshVisiblePainters()
            End If
        End Set
    End Property

    <Category("Appearance"), Description("Gibt den Zwischenraum zwischen den angezeigten Thumbnails zurck oder legt diesen fest.")> _
    Property InnerDistance() As Integer
        Get
            Return m_InnerDistance
        End Get
        Set(ByVal value As Integer)
            Dim TmpValue As Integer = Math.Max(Math.Min(value, 50), 5)
            If m_InnerDistance <> TmpValue Then
                m_InnerDistance = TmpValue
                RefreshVisiblePainters()
            End If
        End Set
    End Property

    ''' <summary>
    ''' legt die angezeigten Pfade fest oder gibt diese zurck
    ''' </summary>
    <Browsable(False)> _
    Property Pfade() As ThumbPath()
        Get
            Return m_Pfade
        End Get
        Set(ByVal value As ThumbPath())
            If value IsNot Nothing Then
                m_Pfade = value

                ' gegebenenfalls existierende Bitmaps frei geben
                For Each Ptr As ThumbPainter In m_Painters
                    Ptr.DisposeBitmap()
                Next
                ' die bisher existierenden ThumbPainter sind berflssig
                m_Painters.Clear()
                m_SelectedPainters.Clear(False)
                m_lastSelectedPainter = Nothing

                ' jetzt fr jeden Pfad einen Thumb-Painter erstellen
                For Each Ref As ThumbPath In m_Pfade
                    m_Painters.Add(New ThumbPainter(Ref, Me))
                Next
                ' nun die ThumbPainter wie im Feld Sortierung festgelegt, sortieren
                Neu_Sortieren()

                ' die ScrollBar auf Null zurck stellen
                Me.VerticalScroll.Value = 0
                ' Painter-Positionen und sichtbare Painter neu festlegen und Anzeige aktualisieren
                RefreshVisiblePainters()
            End If
        End Set
    End Property

    ''' <summary>
    ''' legt die selektierten Pfade fest oder gibt diese zurck
    ''' </summary>
    <Browsable(False)> _
    Property SelectedPfade() As ThumbPath()
        Get
            Dim Result As New List(Of ThumbPath)
            For Each Ptr As ThumbPainter In m_SelectedPainters
                Result.Add(Ptr.Ressource)
            Next
            Return Result.ToArray
        End Get
        Set(ByVal value As ThumbPath())
            If value IsNot Nothing Then
                Dim NewSelectedPainters As New List(Of ThumbPainter)
                For Each Path As ThumbPath In value
                    For Each Ptr As ThumbPainter In m_Painters
                        If Ptr.Ressource Is Path Then
                            NewSelectedPainters.Add(Ptr)
                            Exit For
                        End If
                    Next
                Next
                m_SelectedPainters.SetSelectedItems(NewSelectedPainters)
            End If
        End Set
    End Property

#End Region

#Region " Konstruktor "

    Public Sub New()
        ' ScrollBar-Breite ermitteln
        Dim TmpScrollBar As New VScrollBar
        m_ScrollBarWidth = TmpScrollBar.Width
        TmpScrollBar.Dispose()

        Me.BackColor = Color.White

        ' pen zum Selektieren
        m_SelectionPen = New Pen(SystemColors.Highlight, 4)
        ' DoubleBuffering aktivieren
        Me.SetStyle(ControlStyles.DoubleBuffer Or ControlStyles.UserPaint Or ControlStyles.AllPaintingInWmPaint Or ControlStyles.UserMouse, True)

        ReDim m_Pfade(-1)
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' Sortiert die angezeigten Thumb-Painter neu
    ''' </summary>
    Private Sub Neu_Sortieren()
        Select Case m_Sortierung
            Case ThumbSortierTypen.Normal
                Dim Pfade As New List(Of ThumbPath)
                Pfade.AddRange(m_Pfade)
                m_Painters.Sort(New ThumbNormalComparer(Pfade))
            Case ThumbSortierTypen.Datum
                m_Painters.Sort(New ThumbDatumComparer)
            Case ThumbSortierTypen.Groesse
                m_Painters.Sort(New ThumbGroesseComparer)
        End Select
    End Sub

    ''' <summary>
    ''' zeigt den Inhalt nach nderung der Directory-Auswahl neu an
    ''' </summary>
    Private Sub DisplayNewPath()

        ' gegebenenfalls existierende Bitmaps frei geben
        For Each Ptr As ThumbPainter In m_Painters
            Ptr.DisposeBitmap()
        Next
        ' die bisher existierenden ThumbPainter sind berflssig
        m_Painters.Clear()
        m_SelectedPainters.Clear(False)
        m_lastSelectedPainter = Nothing

        ' Try-Catch, weil beim Versuch, auf ein Directory zuzugreifen, z.B. IO-Fehler passieren knnen
        Try
            ' Liste mit Dateien erstellen
            Dim Dateien() As String = IO.Directory.GetFiles(m_Path)
            ' mit Pfad.Create das Erstellen neuer Thumbnails beginnen
            m_Pfade = ThumbPath.Create(Dateien)
            ' und jetzt fr jeden Pfad einen Thumb-Painter erstellen
            For Each Ref As ThumbPath In m_Pfade
                m_Painters.Add(New ThumbPainter(Ref, Me))
            Next
            ' nun die ThumbPainter wie im Feld Sortierung festgelegt, sortieren
            Neu_Sortieren()
        Catch ex As Exception

        End Try

        ' die ScrollBar auf Null zurck stellen
        Me.VerticalScroll.Value = 0
        ' Painter-Positionen und sichtbare Painter neu festlegen und Anzeige aktualisieren
        RefreshVisiblePainters()
    End Sub


    ''' <summary>
    ''' berschreiben von OnPaint, um Inhalte des Controls neu zeichnen
    ''' </summary>
    Protected Overrides Sub OnPaint(ByVal e As System.Windows.Forms.PaintEventArgs)
        MyBase.OnPaint(e)

        Dim Ft As Font = Me.Font
        Dim FontHeight As Integer = Ft.Height
        Dim ImageSize As New Size(m_ImageWidth, m_ImageWidth)

        For Each Ptr As ThumbPainter In m_VisiblePainters
            If Ptr.Bounds.IntersectsWith(e.ClipRectangle) Then
                Ptr.OnPaint(e, Ft, FontHeight)
                If m_SelectedPainters.Contains(Ptr) Then
                    e.Graphics.DrawRectangle(m_SelectionPen, New Rectangle(Ptr.Bounds.Location, ImageSize))
                End If
            End If
        Next
    End Sub

    ''' <summary>
    ''' berschreiben von OnResize, um Inhalte des Controls neu zu positionieren
    ''' </summary>
    Protected Overrides Sub OnResize(ByVal e As System.EventArgs)
        ' Bei Initialisierung des Controls kann Resize auftreten.
        ' Hier ist m_Pfade auf jeden Fall Nothing
        If m_Pfade Is Nothing Then Exit Sub
        ' Painter-Positionen und sichtbare Painter neu festlegen und Anzeige aktualisieren
        RefreshVisiblePainters()

        MyBase.OnResize(e)
    End Sub

    ''' <summary>
    ''' berschreiben von OnMouseDown, um selektierte Inhalte neu festzulegen
    ''' </summary>
    Protected Overrides Sub OnMouseDown(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseDown(e)

        Dim AcPainter As ThumbPainter = Nothing
        For Each Item As ThumbPainter In m_VisiblePainters
            If Item.Bounds.Contains(e.Location) Then
                AcPainter = Item
                Exit For
            End If
        Next


        Select Case Control.ModifierKeys
            Case Nothing
                If AcPainter IsNot Nothing Then
                    m_SelectedPainters.SetSelectedItem(AcPainter)
                    m_lastSelectedPainter = AcPainter
                Else
                    m_SelectedPainters.Clear(True)
                End If
            Case Keys.Control
                If AcPainter IsNot Nothing Then
                    If m_SelectedPainters.Contains(AcPainter) Then
                        m_SelectedPainters.Remove(AcPainter)
                        m_lastSelectedPainter = Nothing
                    Else
                        m_SelectedPainters.Add(AcPainter)
                        m_lastSelectedPainter = AcPainter
                    End If
                End If
            Case Keys.Shift
                If AcPainter IsNot Nothing Then
                    If m_SelectedPainters.Count = 0 Then
                        m_SelectedPainters.Add(AcPainter)
                        m_lastSelectedPainter = AcPainter
                    Else
                        Dim NewSelectionList As List(Of ThumbPainter) = GetSelectionList(AcPainter)
                        m_SelectedPainters.SetSelectedItems(NewSelectionList)
                    End If
                End If
            Case Keys.Shift Or Keys.Control
                If AcPainter IsNot Nothing Then
                    If m_SelectedPainters.Count = 0 Then
                        m_SelectedPainters.Add(AcPainter)
                        m_lastSelectedPainter = AcPainter
                    Else
                        Dim NewSelectionList As List(Of ThumbPainter) = GetSelectionList(AcPainter)
                        m_SelectedPainters.AddRange(NewSelectionList)
                    End If
                End If
        End Select

    End Sub

    ''' <summary>
    ''' liefert eine Liste mit selektierten Items fr Auswahl eines Bereiches
    ''' </summary>
    ''' <param name="AcPainter">Aktuell ausgewhlter ThumbPainter</param>
    Private Function GetSelectionList(ByVal AcPainter As ThumbPainter) As List(Of ThumbPainter)
        Dim Result As New List(Of ThumbPainter)
        If m_lastSelectedPainter IsNot Nothing AndAlso m_lastSelectedPainter IsNot AcPainter Then
            Dim AcIndex As Integer = m_Painters.IndexOf(AcPainter)
            Dim LastIndex As Integer = m_Painters.IndexOf(m_lastSelectedPainter)
            If AcIndex < LastIndex Then
                For I As Integer = AcIndex To LastIndex
                    Result.Add(m_Painters(I))
                Next
            Else
                For I As Integer = LastIndex To AcIndex
                    Result.Add(m_Painters(I))
                Next
            End If
        End If
        Return Result
    End Function


    ''' <summary>
    ''' ordnet die Painter nach ndern der Gre des Controls oder Scrollen neu an
    ''' </summary>
    Private Sub RefreshVisiblePainters()

        ' Liste mit sichtbaren Paintern erst einmal entleeren
        m_VisiblePainters.Clear()

        ' Neue Positionen fr die Thumbnails bestimmen
        Dim NewPositions As ThumbPositions = GetPositions(m_Painters.Count)

        ' Eigenschaften zur Anzeige der ScrollBars anpassen
        Me.AutoScrollMinSize = New Size(Me.Width - m_ScrollBarWidth, NewPositions.NotwendigeHoehe)
        Me.VerticalScroll.LargeChange = Me.Height
        If NewPositions.NotwendigeHoehe > Me.Height Then
            Me.VerticalScroll.Value = Math.Min(Me.VerticalScroll.Value, NewPositions.NotwendigeHoehe - Me.Height)
        Else
            Me.VerticalScroll.Value = 0
        End If

        ' Egal, ob sich hier was gendert hat oder nicht: 
        ' ein Schleifendurchlauf durch alle vorhandenen Painter braucht nicht viel Rechenzeit.
        ' Also hier die Begrenzungsrechtecke fr die Painter neu festlegen und gleichzeitig die Liste
        ' mit sichtbaren Paintern neu fllen
        Dim ScrollValue As Integer = Me.VerticalScroll.Value
        Dim MyRect As Rectangle = ContentRectangle()
        Dim ItemSize As Size = New Size(ImageWidth, ImageWidth + Me.Font.Height * 2)

        For I As Integer = 0 To NewPositions.Positions.Count - 1
            Dim Ptr As ThumbPainter = m_Painters(I)
            Dim Rect As New Rectangle(New Point(NewPositions.Positions(I).X, NewPositions.Positions(I).Y - ScrollValue), ItemSize)
            Ptr.Bounds = Rect

            If Rect.IntersectsWith(MyRect) Then
                m_VisiblePainters.Add(Ptr)
            Else
                ' fr jetzt unsichtbar werdende Painter kann eine ggf. vorhandene Bitmap-Datei 
                ' entfernt werden
                ' Jetzt habe ich's doch auskommentiert. Dies scheint der Arbeitsspeicher
                ' lieber zu haben, als immer wieder neue Bitmaps zu erstellen
                ' Ptr.DisposeBitmap()
            End If
        Next

        ' die Anzeige des Controls aktualisieren
        Invalidate(MyRect)
    End Sub


    ''' <summary>
    ''' liefert das Control-Rechteck ohne ScrollBar
    ''' </summary>
    Private Function ContentRectangle() As Rectangle
        If Me.VScroll Then
            Return New Rectangle(0, 0, Me.Width - m_ScrollBarWidth, Me.Height)
        Else
            Return New Rectangle(0, 0, Me.Width, Me.Height)
        End If
    End Function

    ''' <summary>
    ''' berschreiben von OnScroll, um Inhalte des Controls neu zu positionieren
    ''' </summary>
    Protected Overrides Sub OnScroll(ByVal se As System.Windows.Forms.ScrollEventArgs)
        MyBase.OnScroll(se)
        ' Painter-Positionen und sichtbare Painter neu festlegen und Anzeige aktualisieren
        RefreshVisiblePainters()
    End Sub

    ''' <summary>
    ''' berschreiben von OnMouseWheel, um Inhalte des Controls neu zu positionieren
    ''' </summary>
    Protected Overrides Sub OnMouseWheel(ByVal e As System.Windows.Forms.MouseEventArgs)
        MyBase.OnMouseWheel(e)
        ' Painter-Positionen und sichtbare Painter neu festlegen und Anzeige aktualisieren
        RefreshVisiblePainters()
    End Sub

    ''' <summary>
    ''' Delegat zum Aufrufen von IsVisible von einem anderen Thread aus
    ''' </summary>
    Private Delegate Function DelgIsVisible(ByVal Painter As ThumbPainter) As Boolean

    ''' <summary>
    ''' liefert Information, ob sich ein ThumbPainter im sichtbaren Bereich des Controls befindet
    ''' </summary>
    Public Function IsVisible(ByVal Painter As ThumbPainter) As Boolean
        If Me.InvokeRequired Then
            ' diese Bedingung ist erfllt, wenn der Aufruf der 
            ' Function IsVisible aus einem fremden Thread erfolgt.
            Dim MyDelegate As New DelgIsVisible(AddressOf IsVisible)
            ' also die Function IsVisible erneut per Invoke aufrufen, damit 
            ' kein threadbergreifender Zugriff auf dieses Control erfolgt.
            Return Me.Invoke(MyDelegate, Painter)
        End If

        Return m_VisiblePainters.Contains(Painter)
    End Function

    ''' <summary>
    ''' Delegat zum Aufrufen von InvalidateRectangle von einem anderen Thread aus
    ''' </summary>
    ''' <param name="Rect">Rectangle-Struktur</param>
    Private Delegate Sub DelgInvalidateRectangle(ByVal Rect As Rectangle)

    ''' <summary>
    ''' Veranlasst das Neuzeichnen einer durch ein Rechteck begrenzten Flche im Control
    ''' </summary>
    ''' <param name="Rect">Rectangle-Struktur</param>
    Public Sub InvalidateRectangle(ByVal Rect As Rectangle)
        If Me.InvokeRequired Then
            ' diese Bedingung ist erfllt, wenn der Aufruf der Sub
            ' InvalidateRectangle aus einem fremden Thread erfolgt. 
            Dim MyDelegate As New DelgInvalidateRectangle(AddressOf InvalidateRectangle)
            ' also die Sub InvalidateRectangle erneut per Invoke aufrufen, damit 
            ' kein threadbergreifender Zugriff auf dieses Control erfolgt.
            Me.Invoke(MyDelegate, Rect)
        Else
            Me.Invalidate(Rect)
        End If
    End Sub

    ''' <summary>
    ''' ermittelt Positionen fr Thumbnails neu
    ''' </summary>
    ''' <param name="ItemCount">Anzahl anzuzeigender Items</param>
    Private Function GetPositions(ByVal ItemCount As Integer) As ThumbPositions
        Dim Result As New ThumbPositions

        Dim Zeilenhoehe As Integer = m_InnerDistance + m_ImageWidth + Me.Font.Height * 2

        ' zuerst feststellen, ob Anzeige der Inhalte auch ohne ScrollBar mglich ist
        ' wenn ja, wird diese Variante verwendet
        Dim SpaltenMitScrollbar As Integer = AnzahlSpalten_Liefern(True)
        Dim SpaltenOhneScrollbar As Integer = AnzahlSpalten_Liefern(False)

        Dim ZeilenOhneScrollBar As Integer = (ItemCount + SpaltenOhneScrollbar - 1) \ SpaltenOhneScrollbar
        Dim HoeheOhneScrollBar As Integer = m_InnerDistance + Zeilenhoehe * ZeilenOhneScrollBar

        ' Nun Spalten- und Zeilenanzahl und die Zeilen- bzw. Spaltenpositionen festlegen
        Dim AnzahlSpalten, AnzahlZeilen As Integer

        If HoeheOhneScrollBar > Me.Height Then
            ' Im Control muss eine ScrollBar angezeigt werden
            AnzahlSpalten = SpaltenMitScrollbar
        Else
            ' Im Control ist keine ScrollBar notwendig
            AnzahlSpalten = SpaltenOhneScrollbar
        End If


        AnzahlZeilen = (ItemCount + AnzahlSpalten - 1) \ AnzahlSpalten
        ' die NotwendigeHoehe enthlt das spter bentigte Maximum fr die ScrollBar
        Result.NotwendigeHoehe = m_InnerDistance + Zeilenhoehe * AnzahlZeilen

        Dim ColumnPositions(AnzahlSpalten - 1) As Integer
        Dim SpaltenBreite As Integer = m_ImageWidth + m_InnerDistance
        For I As Integer = 0 To AnzahlSpalten - 1
            ColumnPositions(I) = m_InnerDistance + SpaltenBreite * I
        Next

        Dim RowPositions(AnzahlZeilen - 1) As Integer
        For I As Integer = 0 To AnzahlZeilen - 1
            RowPositions(I) = m_InnerDistance + Zeilenhoehe * I
        Next

        ' jezt kann die Ergebnisliste mit Points aufgefllt werden
        Dim ZeilenIndex As Integer = 0
        Dim SpaltenIndex As Integer = 0
        If ItemCount > 0 Then
            For I As Integer = 0 To ItemCount - 1
                Result.Positions.Add(New Point(ColumnPositions(SpaltenIndex), RowPositions(ZeilenIndex)))
                SpaltenIndex += 1

                If SpaltenIndex = AnzahlSpalten Then
                    ZeilenIndex += 1
                    SpaltenIndex = 0
                End If
            Next
        End If

        ' Ergebnis zurck geben
        Return Result
    End Function

    ''' <summary>
    ''' Hilfsfunktion zum Ermitteln der maximal mglichen Spaltenanzahl abhngig von Breite und
    ''' anderen Einstellungen dieses Controls
    ''' </summary>
    Private Function AnzahlSpalten_Liefern(ByVal SrollBarBeruecksichtigt As Boolean) As Integer

        Dim RestBreite As Integer = Me.Width - m_InnerDistance
        If SrollBarBeruecksichtigt Then RestBreite -= m_ScrollBarWidth

        Return Math.Max(RestBreite \ (m_InnerDistance + m_ImageWidth), 1)
    End Function

    ''' <summary>
    ''' Handler fr das Changed-Event der SelectedPainters-Liste
    ''' </summary>
    Private Sub m_SelectedPainters_Changed() Handles m_SelectedPainters.Changed
        ' Control zum Neuzeichnen zwingen
        Invalidate()
    End Sub
#End Region

#Region " Private Hilfsklassen "

#Region " Comparer zum Sortieren "
    Private Class ThumbNormalComparer
        Implements IComparer(Of ThumbPainter)
        Private m_Pfade As List(Of ThumbPath)
        Public Sub New(ByVal Pfade As List(Of ThumbPath))
            m_Pfade = Pfade
        End Sub
        Public Function Compare(ByVal x As ThumbPainter, ByVal y As ThumbPainter) As Integer Implements System.Collections.Generic.IComparer(Of ThumbPainter).Compare
            Return m_Pfade.IndexOf(x.Ressource).CompareTo(m_Pfade.IndexOf(y.Ressource))
        End Function
    End Class
    Private Class ThumbDatumComparer
        Implements IComparer(Of ThumbPainter)

        Public Function Compare(ByVal x As ThumbPainter, ByVal y As ThumbPainter) As Integer Implements System.Collections.Generic.IComparer(Of ThumbPainter).Compare
            Return x.FileDate.CompareTo(y.FileDate)
        End Function
    End Class
    Private Class ThumbGroesseComparer
        Implements IComparer(Of ThumbPainter)

        Public Function Compare(ByVal x As ThumbPainter, ByVal y As ThumbPainter) As Integer Implements System.Collections.Generic.IComparer(Of ThumbPainter).Compare
            Return x.FileSize.CompareTo(y.FileSize)
        End Function
    End Class
#End Region

#Region " ThumbPositions "

    ''' <summary>
    ''' liefert Positionen fr Thumbnails und Notwendige Hhe des Thumbcontrols
    ''' </summary>
    Private Class ThumbPositions
        ' Notwendige Hhe, die im Control anzeigbar sein muss
        Private m_NotwendigeHoehe As Integer = 0
        ' Liste mit Points, an denen die einzelnen Thumbnails bei Nullposition
        ' der ScrollBar angezeigt werden
        Private m_Positions As New List(Of Point)

        ''' <summary>
        ''' Notwendige Hhe, die im Control anzeigbar sein muss
        ''' </summary>
        Property NotwendigeHoehe() As Integer
            Get
                Return m_NotwendigeHoehe
            End Get
            Set(ByVal value As Integer)
                m_NotwendigeHoehe = value
            End Set
        End Property

        ''' <summary>
        ''' Liste mit Points, die die Positionen der Thumbnails bei Nullposition 
        ''' der ScrollBar enthalten
        ''' </summary>
        ReadOnly Property Positions() As List(Of Point)
            Get
                Return m_Positions
            End Get
        End Property

    End Class

#End Region

#End Region

End Class
