''' <summary>
''' zeichnet ein einzelnes Thumbnail-Element im Thumb-Control
''' </summary>
Public Class ThumbPainter

#Region " Private Felder "
    ' gemeinsam genutztes StringFormat
    Private Shared m_SF As StringFormat

    ' Pfad-Source fr Thumbnail
    Private WithEvents m_Ressource As ThumbPath
    ' Rechteck, das Position und Begrenzung enthlt
    Private m_Bounds As Rectangle

    ' Dateiname
    Private m_FileName As String
    ' Letzter Schreibzugriff 
    Private m_FileDate As Date
    ' Datei-Gre
    Private m_FileSize As Long
    ' Bitmap zum Zwischnespeichern des Thumbnails
    Private m_Bitmap As Bitmap = Nothing
    ' Referenz zum Parent-Control
    Private m_Ctl As ThumbControl
#End Region

#Region " Properties "

    ''' <summary>
    ''' Datei-Gre
    ''' </summary>
    ReadOnly Property FileSize() As Long
        Get
            Return m_FileSize
        End Get
    End Property

    ''' <summary>
    ''' Referenz zur Thumb-Source
    ''' </summary>
    ReadOnly Property Ressource() As ThumbPath
        Get
            Return m_Ressource
        End Get
    End Property

    ''' <summary>
    ''' Dateiname
    ''' </summary>
    ReadOnly Property FileName() As String
        Get
            Return m_FileName
        End Get
    End Property


    ''' <summary>
    ''' letzter Schreibzugriff
    ''' </summary>
    ReadOnly Property FileDate() As Date
        Get
            Return m_FileDate
        End Get
    End Property


    ''' <summary>
    ''' Rechteck, das Position und Begrenzung enthlt
    ''' </summary>
    Property Bounds() As Rectangle
        Get
            Return m_Bounds
        End Get
        Set(ByVal value As Rectangle)
            m_Bounds = value
        End Set
    End Property

#End Region

#Region " Konstruktor "

    Shared Sub New()
        ' Gemeinsam genutzes Stringformat festlegen
        m_SF = New System.Drawing.StringFormat
        m_SF.Trimming = StringTrimming.EllipsisCharacter
    End Sub

    Public Sub New(ByVal Pfad As ThumbPath, ByVal Ctl As ThumbControl)
        m_Ctl = Ctl
        m_Ressource = Pfad
        Dim FI As New IO.FileInfo(m_Ressource.ToString)
        m_FileName = FI.Name
        m_FileDate = FI.LastWriteTime
        m_FileSize = FI.Length
    End Sub

#End Region

#Region " Methoden "

    ''' <summary>
    ''' zeichnet den Inhalt im ThumbPainter neu
    ''' </summary>
    ''' <param name="e">PaintEventArgs</param>
    ''' <param name="Ft">Font</param>
    ''' <param name="FontHeight">FontHeight</param>
    Overridable Sub OnPaint(ByVal e As PaintEventArgs, ByVal Ft As Font, ByVal FontHeight As Integer)
        Dim gr As Graphics = e.Graphics

        Dim MyRect As Rectangle = m_Bounds
        Dim Width As Integer = m_Bounds.Width

        If m_Bitmap Is Nothing Then
            ' Wenn keine Bitmap existiert, eine neue erstellen
            m_Bitmap = New Bitmap(Width, Width, Drawing.Imaging.PixelFormat.Format32bppArgb)
            ' Graphics-Objekt aus Bitmap erstellen und darauf zeichnen
            Using g As Graphics = Graphics.FromImage(m_Bitmap)
                Dim InnerRect As New Rectangle(1, 1, Width - 2, Width - 2)
                Using InnerBrush As New Drawing2D.LinearGradientBrush(InnerRect, Color.White, Color.LightGray, Drawing2D.LinearGradientMode.ForwardDiagonal)
                    ' Flche mit grauem Gradienten ausfllen
                    g.FillRectangle(InnerBrush, InnerRect)
                End Using
                ' dunkelgrauen Rahmen zeichnen
                g.DrawRectangle(Pens.LightGray, 0, 0, Width - 1, Width - 1)

                If m_Ressource.HasImage Then
                    ' Wenn ein Image zum Pfad existiert
                    If m_Ressource.IsImagePath Then
                        ' wenn der Pfad ein Bild-Pfad ist
                        Dim Path As String = ThumbPath.GetThumbPath(m_Ressource.ToString)
                        Using Img As Image = Image.FromFile(Path)
                            ' ggf. verkleinerte Kantenlnge fr das zu zeichnende Bild ermitteln
                            Dim ZoomedSize As Size
                            If Img.Width > Width - 2 OrElse Img.Height > Width - 2 Then
                                ZoomedSize = GetPictureSize(New Size(Width - 2, Width - 2), Img.Size)
                            Else
                                ZoomedSize = Img.Size
                            End If
                            Dim Left As Integer = (Width - ZoomedSize.Width) \ 2
                            Dim Top As Integer = (Width - ZoomedSize.Height) \ 2
                            ' Bild am ermittelten Platz mit ermittelnder Kantenlnge zeichnen
                            g.DrawImage(Img, Left, Top, ZoomedSize.Width, ZoomedSize.Height)
                        End Using
                    Else
                        ' ansonsten Datei-Icon zeichnen
                        Dim IconWidth As Integer = Width / 100 * 32
                        Dim Left As Integer = (Width - IconWidth) \ 2
                        Dim Icn As Icon = Icon.ExtractAssociatedIcon(m_Ressource.ToString)
                        g.InterpolationMode = Drawing2D.InterpolationMode.NearestNeighbor
                        g.DrawIcon(Icn, New Rectangle(Left, Left, IconWidth, IconWidth)) 'Left, Top)
                    End If
                End If
            End Using
        End If

        ' die zwischengespeicherte Bitmap zeichnen
        gr.DrawImage(m_Bitmap, New Rectangle(m_Bounds.Location, New Size(Width, Width)))

        ' Beschriftung zeichnen
        Dim TextRect As New RectangleF(MyRect.X + 1, MyRect.Y + Width + 2, Width - 2, FontHeight)
        gr.DrawString(m_FileName, Ft, Brushes.Black, TextRect, m_SF)
        TextRect.Offset(0.0F, FontHeight)
        gr.DrawString(m_FileDate.ToShortDateString & " " & m_FileDate.ToShortTimeString, Ft, Brushes.Gray, TextRect, m_SF)

    End Sub

    Private Function GetIconSize(ByVal ThumbnailWidth As Integer) As Integer
        Return ThumbnailWidth \ 100 * 32
    End Function


    ''' <summary>
    ''' Liefert die passende verkleinerte Kantenlnge zum Zeichnen einer Thumbnail-Datei 
    ''' in einem Graphics-Objekt
    ''' </summary>
    ''' <param name="DestinationSize">maximale Zielgre</param>
    ''' <param name="PicSize">Originalgre der Thumbnail-Datei</param>
    Private Function GetPictureSize(ByVal DestinationSize As Size, ByVal PicSize As Size) As Size
        Dim pw As Single = PicSize.Width
        Dim ph As Single = PicSize.Height
        Dim dw As Single = DestinationSize.Width
        Dim dh As Single = DestinationSize.Height

        Dim scaleFactor As Single
        If pw / ph > dw / dh Then
            scaleFactor = dw / pw
        Else
            scaleFactor = dh / ph
        End If

        Dim Width As Integer = Math.Min(scaleFactor * pw, DestinationSize.Width)
        Dim Height As Integer = Math.Min(scaleFactor * ph, DestinationSize.Height)

        Return New Size(Width, Height)
    End Function

    ''' <summary>
    ''' entfernt die Bitmap mit dem Thumbnail und gibt Ressourcen frei
    ''' </summary>
    Public Sub DisposeBitmap()
        If m_Bitmap IsNot Nothing Then
            m_Bitmap.Dispose()
            m_Bitmap = Nothing
        End If
    End Sub

    ''' <summary>
    ''' Handler fr das Image_Changed-Ereignis der ThumbRessource
    ''' </summary>
    Private Sub m_Ressource_Image_Changed() Handles m_Ressource.Image_Changed
        m_Bitmap = Nothing
        If m_Ctl.IsVisible(Me) Then
            ' wenn dieser Thumb-Painter auf im ThumbControl gerade sichtbar ist, 
            ' wird das Control zum Neuzeichnen gezwungen
            m_Ctl.InvalidateRectangle(Me.Bounds)
        End If
    End Sub

#End Region

End Class


